import type { KeyIndex, PrivateKeyData } from "../rpc";
import { getWax } from "../hive/wax";
import { remove0x } from "@metamask/utils";
import { keyIndexToPath } from "../utils/key-management";
import { getTempWallet } from "../hive/beekeeper";
import { ConfirmGetPrivateKeys } from "./dialogs/ConfirmGetPrivateKeys";
import { SLIP10Node } from "@metamask/key-tree";

export const getPrivateKeys = async (origin: string, keys: KeyIndex[]): Promise<PrivateKeyData[]> => {
  if (keys.length < 1)
    throw new Error('No keys provided');

  const confirmSign = await ConfirmGetPrivateKeys(origin, keys);

  if(!confirmSign)
    throw new Error('User denied the transaction');

  // The order is important: First create wax, then transaction and if all success then create wallet
  const wax = await getWax();
  const wallet = await getTempWallet();

  try {
    const privateKeys: PrivateKeyData[] = [];

    for(const key of keys) {
      const snapResponse = await snap.request({
        method: 'snap_getBip32Entropy',
        params: {
          curve: "secp256k1",
          path: keyIndexToPath(key)
        }
      });

      const node = await SLIP10Node.fromJSON(snapResponse);

      if (!node.privateKey)
        throw new Error('No private key found');

      const wif = wax.convertRawPrivateKeyToWif(remove0x(node.privateKey));

      const publicKey = await wallet.importKey(wif);

      privateKeys.push({
        accountIndex: key.accountIndex ?? 0,
        role: key.role,
        privateKey: wif,
        publicKey
      });
    }

    return privateKeys;
  } finally {
    wallet.close();
  }
};
